@@ -3,18 +3,17 @@ use std::convert::{FloatToInt, TryFrom, TryInto};
 use std::fmt::{Debug, Display, Formatter};
 use std::ops::{Add, AddAssign, Div, Mul, MulAssign, Neg, Sub, SubAssign};
 
-use crate::big_int::BigInt;
-use crate::digits::{
-    AdditiveDigit, DigitConvertibleFromFloat, EuclidDivisibleDigit, GcdDigit, LeftShiftableDigit,
-    MultiplicativeDigit, UnitaryDigit,
+use crate::big_int::{
+    AdditiveDigit, BigInt, DigitConvertibleFromF64, EuclidDivisibleDigit, GcdDigit,
+    LeftShiftableDigit, MultiplicativeDigit,
 };
+use crate::contracts::is_signed;
 use crate::traits::{
     Abs, AdditiveMonoid, Ceil, CheckedDiv, CheckedDivAsF32, CheckedDivAsF64, CheckedDivEuclid,
     CheckedDivRemEuclid, CheckedPow, CheckedRemEuclid, CheckedShl, DivisivePartialMagma, Float,
     Floor, GcdMagma, Maybe, ModularUnaryAlgebra, MultiplicativeMonoid, NegatableUnaryAlgebra,
     Oppositive, Parity, Pow, Round, SubtractiveMagma, TieBreaking, Trunc, Unitary, Zeroable,
 };
-use crate::utils;
 
 #[derive(Clone, Eq, PartialEq)]
 pub struct Fraction<Component: Clone + Eq> {
@@ -50,6 +49,15 @@ impl Display for FromFloatConversionError {
     }
 }
 
+/// Constructs a new fraction from given numerator and denominator.
+/// Examples:
+/// ```rust
+/// use rithm::fraction;
+/// type Fraction = fraction::Fraction<i8>;
+/// assert_eq!(Fraction::new(1, 2), Some(Fraction::from(1) / 2));
+/// assert_eq!(Fraction::new(2, 4), Fraction::new(1, 2));
+/// assert_eq!(Fraction::new(-1, -2), Fraction::new(1, 2));
+/// ```
 impl<Component: Clone + DivisivePartialMagma + Eq + GcdMagma + Oppositive> Fraction<Component> {
     pub fn new(mut numerator: Component, mut denominator: Component) -> Option<Self> {
         if denominator.is_zero() {
@@ -143,7 +151,7 @@ impl<
     }
 }
 
-macro_rules! plain_add_fraction_impl {
+macro_rules! primitive_add_fraction_impl {
     ($($t:ty)*) => ($(
     impl Add<Fraction<Self>> for $t {
         type Output = Fraction<Self>;
@@ -155,10 +163,13 @@ macro_rules! plain_add_fraction_impl {
     )*)
 }
 
-plain_add_fraction_impl!(i8 i16 i32 i64 i128 isize);
+primitive_add_fraction_impl!(i8 i16 i32 i64 i128 isize);
 
-impl<Digit: AdditiveDigit + Eq + GcdDigit, const SEPARATOR: char, const SHIFT: usize>
-    Add<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
+impl<
+        Digit: AdditiveDigit + Eq + GcdDigit + MultiplicativeDigit,
+        const SEPARATOR: char,
+        const SHIFT: usize,
+    > Add<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
 {
     type Output = Fraction<Self>;
 
@@ -252,7 +263,7 @@ impl<
     }
 }
 
-macro_rules! plain_checked_div_fraction_impl {
+macro_rules! primitive_checked_div_fraction_impl {
     ($($t:ty)*) => ($(
     impl CheckedDiv<Fraction<Self>> for $t
     {
@@ -274,7 +285,7 @@ macro_rules! plain_checked_div_fraction_impl {
     )*)
 }
 
-plain_checked_div_fraction_impl!(i8 i16 i32 i64 i128 isize);
+primitive_checked_div_fraction_impl!(i8 i16 i32 i64 i128 isize);
 
 impl<Digit: Eq + GcdDigit + MultiplicativeDigit, const SEPARATOR: char, const SHIFT: usize>
     CheckedDiv<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
@@ -319,7 +330,7 @@ impl<
     }
 }
 
-macro_rules! plain_checked_div_euclid_fraction_impl {
+macro_rules! primitive_checked_div_euclid_fraction_impl {
     ($($t:ty)*) => ($(
     impl CheckedDivEuclid<Fraction<Self>> for $t
     {
@@ -332,7 +343,7 @@ macro_rules! plain_checked_div_euclid_fraction_impl {
     )*)
 }
 
-plain_checked_div_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+primitive_checked_div_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
 impl<
         Digit: Eq + EuclidDivisibleDigit + MultiplicativeDigit,
@@ -403,7 +414,7 @@ impl<
     }
 }
 
-macro_rules! plain_checked_div_rem_euclid_fraction_impl {
+macro_rules! primitive_checked_div_rem_euclid_fraction_impl {
     ($($t:ty)*) => ($(
     impl CheckedDivRemEuclid<Fraction<Self>> for $t
     {
@@ -427,7 +438,7 @@ macro_rules! plain_checked_div_rem_euclid_fraction_impl {
     )*)
 }
 
-plain_checked_div_rem_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+primitive_checked_div_rem_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
 impl<
         Digit: Eq + EuclidDivisibleDigit + GcdDigit + MultiplicativeDigit,
@@ -491,7 +502,7 @@ impl<
     }
 }
 
-macro_rules! plain_fraction_checked_pow_impl {
+macro_rules! primitive_fraction_checked_pow_impl {
     ($($t:ty)*) => ($(
         impl CheckedPow<u32> for Fraction<$t> {
             type Output = Option<Self>;
@@ -506,7 +517,7 @@ macro_rules! plain_fraction_checked_pow_impl {
         )*)
 }
 
-plain_fraction_checked_pow_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+primitive_fraction_checked_pow_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
 impl<
         Component: Clone
@@ -558,7 +569,7 @@ impl<
     }
 }
 
-macro_rules! plain_checked_rem_euclid_fraction_impl {
+macro_rules! primitive_checked_rem_euclid_fraction_impl {
     ($($t:ty)*) => ($(
         impl CheckedRemEuclid<Fraction<Self>> for $t
         {
@@ -583,7 +594,7 @@ macro_rules! plain_checked_rem_euclid_fraction_impl {
         )*)
 }
 
-plain_checked_rem_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+primitive_checked_rem_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
 
 impl<
         Digit: Eq + EuclidDivisibleDigit + GcdDigit + MultiplicativeDigit,
@@ -662,7 +673,7 @@ impl<
     }
 }
 
-macro_rules! plain_div_fraction_impl {
+macro_rules! primitive_div_fraction_impl {
     ($($t:ty)*) => ($(
     impl Div<Fraction<Self>> for $t
     {
@@ -675,7 +686,7 @@ macro_rules! plain_div_fraction_impl {
     )*)
 }
 
-plain_div_fraction_impl!(i8 i16 i32 i64 i128 isize);
+primitive_div_fraction_impl!(i8 i16 i32 i64 i128 isize);
 
 impl<Digit: Eq + GcdDigit + MultiplicativeDigit, const SEPARATOR: char, const SHIFT: usize>
     Div<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
@@ -735,7 +746,7 @@ impl<
     }
 }
 
-macro_rules! plain_mul_fraction_impl {
+macro_rules! primitive_mul_fraction_impl {
     ($($t:ty)*) => ($(
     impl Mul<Fraction<Self>> for $t {
         type Output = Fraction<Self>;
@@ -747,7 +758,7 @@ macro_rules! plain_mul_fraction_impl {
     )*)
 }
 
-plain_mul_fraction_impl!(i8 i16 i32 i64 i128 isize);
+primitive_mul_fraction_impl!(i8 i16 i32 i64 i128 isize);
 
 impl<Digit: Eq + GcdDigit + MultiplicativeDigit, const SEPARATOR: char, const SHIFT: usize>
     Mul<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
@@ -812,7 +823,7 @@ impl<Component: Clone + Eq + Unitary> PartialEq<Component> for Fraction<Componen
     }
 }
 
-macro_rules! plain_partial_eq_fraction_impl {
+macro_rules! primitive_partial_eq_fraction_impl {
     ($($t:ty)*) => ($(
     impl PartialEq<Fraction<Self>> for $t {
         fn eq(&self, other: &Fraction<Self>) -> bool {
@@ -822,9 +833,9 @@ macro_rules! plain_partial_eq_fraction_impl {
     )*)
 }
 
-plain_partial_eq_fraction_impl!(i8 i16 i32 i64 i128 isize);
+primitive_partial_eq_fraction_impl!(i8 i16 i32 i64 i128 isize);
 
-impl<Digit: Clone + Eq + UnitaryDigit, const SEPARATOR: char, const SHIFT: usize>
+impl<Digit: Clone + Eq + Unitary, const SEPARATOR: char, const SHIFT: usize>
     PartialEq<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
 {
     fn eq(&self, other: &Fraction<Self>) -> bool {
@@ -894,24 +905,24 @@ impl<Component: Clone + Eq + MultiplicativeMonoid + PartialOrd> PartialOrd<Compo
     }
 }
 
-macro_rules! plain_partial_ord_fraction_impl {
+macro_rules! primitive_partial_ord_fraction_impl {
     ($($t:ty)*) => ($(
     impl PartialOrd<Fraction<Self>> for $t
     {
         fn ge(&self, other: &Fraction<Self>) -> bool {
-            self.clone() * other.denominator.clone() >= other.numerator
+            self * other.denominator >= other.numerator
         }
 
         fn gt(&self, other: &Fraction<Self>) -> bool {
-            self.clone() * other.denominator.clone() > other.numerator
+            self * other.denominator > other.numerator
         }
 
         fn le(&self, other: &Fraction<Self>) -> bool {
-            self.clone() * other.denominator.clone() <= other.numerator
+            self * other.denominator <= other.numerator
         }
 
         fn lt(&self, other: &Fraction<Self>) -> bool {
-            self.clone() * other.denominator.clone() < other.numerator
+            self * other.denominator < other.numerator
         }
 
         fn partial_cmp(&self, other: &Fraction<Self>) -> Option<Ordering> {
@@ -927,7 +938,7 @@ macro_rules! plain_partial_ord_fraction_impl {
     )*)
 }
 
-plain_partial_ord_fraction_impl!(i8 i16 i32 i64 i128 isize);
+primitive_partial_ord_fraction_impl!(i8 i16 i32 i64 i128 isize);
 
 impl<
         Digit: Clone + Eq + GcdDigit + MultiplicativeDigit + PartialOrd,
@@ -1081,7 +1092,7 @@ impl<
     }
 }
 
-macro_rules! plain_sub_fraction_impl {
+macro_rules! primitive_sub_fraction_impl {
     ($($t:ty)*) => ($(
     impl Sub<Fraction<Self>> for $t {
         type Output = Fraction<Self>;
@@ -1100,10 +1111,13 @@ macro_rules! plain_sub_fraction_impl {
     )*)
 }
 
-plain_sub_fraction_impl!(i8 i16 i32 i64 i128 isize);
+primitive_sub_fraction_impl!(i8 i16 i32 i64 i128 isize);
 
-impl<Digit: AdditiveDigit + Eq + GcdDigit, const SEPARATOR: char, const SHIFT: usize>
-    Sub<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
+impl<
+        Digit: AdditiveDigit + Eq + GcdDigit + MultiplicativeDigit,
+        const SEPARATOR: char,
+        const SHIFT: usize,
+    > Sub<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
 {
     type Output = Fraction<Self>;
 
@@ -1212,7 +1226,7 @@ impl<Component: Clone + Eq + Unitary + Zeroable> Zeroable for Fraction<Component
 macro_rules! big_int_try_from_float_impl {
     ($($f:ty)*) => ($(
         impl<
-                Digit: DigitConvertibleFromFloat + Eq + LeftShiftableDigit + UnitaryDigit,
+                Digit: DigitConvertibleFromF64 + Eq + LeftShiftableDigit + Unitary,
                 const SEPARATOR: char,
                 const SHIFT: usize,
             > TryFrom<$f> for Fraction<BigInt<Digit, SEPARATOR, SHIFT>>
@@ -1255,7 +1269,7 @@ macro_rules! big_int_try_from_float_impl {
 
 big_int_try_from_float_impl!(f32 f64);
 
-macro_rules! plain_try_from_float_impl {
+macro_rules! primitive_try_from_float_impl {
     ($f:ty => $($t:ty)*) => ($(
         impl TryFrom<$f> for Fraction<$t> {
             type Error = FromFloatConversionError;
@@ -1269,7 +1283,7 @@ macro_rules! plain_try_from_float_impl {
                     Err(FromFloatConversionError::OutOfBounds)
                 } else {
                     let (mut fraction, mut exponent) = value.frexp();
-                    const MAX_EXPONENT_MODULUS: u32 = <$t>::BITS - 1 - (utils::is_signed::<$t>() as u32);
+                    const MAX_EXPONENT_MODULUS: u32 = <$t>::BITS - 1 - (is_signed::<$t>() as u32);
                     if (exponent.abs() as u32) > MAX_EXPONENT_MODULUS {
                         if exponent.is_negative() {
                             fraction *= ((exponent + (MAX_EXPONENT_MODULUS as i32)) as $f).exp2();
@@ -1308,8 +1322,8 @@ macro_rules! plain_try_from_float_impl {
     )*)
 }
 
-plain_try_from_float_impl!(f32 => i8 i16 i32 i64 u8 u16 u32 u64);
-plain_try_from_float_impl!(f64 => i8 i16 i32 i64 u8 u16 u32 u64);
+primitive_try_from_float_impl!(f32 => i8 i16 i32 i64 u8 u16 u32 u64);
+primitive_try_from_float_impl!(f64 => i8 i16 i32 i64 u8 u16 u32 u64);
 
 #[inline]
 fn normalize_components_moduli<Component: Clone + DivisivePartialMagma + GcdMagma>(
