@@ -31,6 +31,12 @@ type Digit = u16;
 type Digit = u32;
 const BINARY_SHIFT: usize = (Digit::BITS - 1) as usize;
 const _: () = assert!(big_int::is_valid_shift::<Digit, BINARY_SHIFT>());
+#[cfg(target_arch = "x86")]
+const HASH_BITS: usize = 31;
+#[cfg(not(target_arch = "x86"))]
+const HASH_BITS: usize = 61;
+const HASH_INF: ffi::Py_hash_t = 314159;
+const HASH_MODULUS: usize = (1 << HASH_BITS) - 1;
 const PICKLE_SERIALIZATION_ENDIANNESS: Endianness = Endianness::Little;
 
 type BigInt = big_int::BigInt<Digit, '_', BINARY_SHIFT>;
@@ -886,6 +892,36 @@ impl PyFraction {
             .to_object(py)
     }
 
+    fn __hash__(&self) -> ffi::Py_hash_t {
+        let inverted_denominator = unsafe {
+            self.0
+                .denominator()
+                .checked_pow_rem_euclid(BigInt::from(HASH_MODULUS - 2), BigInt::from(HASH_MODULUS))
+                .unwrap_unchecked()
+        };
+        let result = if inverted_denominator.is_zero() {
+            HASH_INF
+        } else {
+            unsafe {
+                ffi::Py_hash_t::try_from(
+                    (self.0.numerator().abs() * inverted_denominator)
+                        .checked_rem_euclid(BigInt::from(HASH_MODULUS))
+                        .unwrap_unchecked(),
+                )
+                .unwrap_unchecked()
+            }
+        };
+        if self.0.is_negative() {
+            if result.is_one() {
+                -2
+            } else {
+                -result
+            }
+        } else {
+            result
+        }
+    }
+
     fn __mod__(&self, divisor: &PyAny) -> PyResult<PyObject> {
         let py = divisor.py();
         if divisor.is_instance(PyFraction::type_object(py))? {
@@ -1132,11 +1168,6 @@ impl PyFraction {
 }
 
 fn hash(value: &BigInt) -> usize {
-    #[cfg(target_arch = "x86")]
-    const HASH_BITS: usize = 31;
-    #[cfg(not(target_arch = "x86"))]
-    const HASH_BITS: usize = 61;
-    const HASH_MODULUS: usize = (1 << HASH_BITS) - 1;
     if value.digits().len() == 1 {
         return if value.is_negative() {
             usize::MAX
@@ -1160,7 +1191,11 @@ fn hash(value: &BigInt) -> usize {
     if value.is_negative() {
         result = usize::MAX - result + 1
     };
-    result - ((result == usize::MAX) as usize)
+    if result == usize::MAX {
+        result - 1
+    } else {
+        result
+    }
 }
 
 fn compare<T: PartialOrd<U>, U>(left: &T, right: &U, op: CompareOp) -> bool {
